#!/bin/sh

# This script detects the installed HBAs
# and accordingly modify the /etc/modprobe.conf (RHEL)
# or /etc/sysconfig/kernel (SLES)
# This script is used as POST_INSTALL script.
# Most of the code here is taken from QL installer

QL_FC_ISP_LIST=( 2300 2312 2322 6312 6322 2400 2422 2432 5422 5432 )
QL_HBA_LIST=( ) # Array of all the HBAs installed in the system
QL_FC_HBA_LIST=( ) # Persistent list of FC HBAs
QL_HBA_MODULES=( ) # ISP to modules converted array

QL_SUCCESS=0
QL_FAIL=1

KERNEL_VERSION=`uname -r`
KERNEL_MAJ_MIN=`echo $KERNEL_VERSION | cut -d . -f -2` 

SLES=/etc/SuSE-release
SYS_FILE=kernel
SLES_CONF=/etc/sysconfig/$SYS_FILE
SLES_CONF_KEY=INITRD_MODULES
RHEL=/etc/redhat-release

SLES_MODPROBE="modprobe.conf.local"
SLES_MPC=/etc/$SLES_MODPROBE
RHEL_MODPROBE="modprobe.conf"
RHEL_MPC=/etc/$RHEL_MODPROBE

if [ "${KERNEL_MAJ_MIN}" = "2.4" ]; then
	RHEL_MODPROBE="modules.conf"
	RHEL_MPC=/etc/$RHEL_MODPROBE
fi

QL_MODULE_VER=qla2xxx-8.01.05
QL_MOD_DIR="/usr/src/$QL_MODULE_VER"
QL_CONF_ENTRIES="$QL_MOD_DIR/ql-conf-entries"
QL_OPTIONS_ENTRIES="$QL_MOD_DIR/ql-opt-entries"
QL_LINE_ENTRIES="$QL_MOD_DIR/qla2-entries"
QL_RENAMES_FILE="/etc/modprobe.d/module-renames"
QL_RENAMES_FILE_BKUP="$QL_MOD_DIR/module-renames-ql.bak"
QL_RENAMES_FILE_DATE="$QL_MOD_DIR/module-renames-ql"

# --------------------------------------------------------- #
# detect_isp () :                                          #
# Detects if the installed HBA is of typed FC or iSCSI      #
# Fills up the global QL_HBA_LIST array with the installed  #
# HBAs                                                      #
# Parameter:                                                #
#       $1: Array of FC or iSCSI ISPs to be detected        #
# Returns: QL_SUCCESS if HBA found. QL_FAIL if no HBA found #
# --------------------------------------------------------- #
function detect_isp() {
        ISP_LIST=( `echo "$1"` )
        local ret_status=$QL_FAIL
        QL_HBA_LIST=() #Empty array
        # Get the list of line which has the keyword qlogic
        # Assumed that qlogic will be there in the output
        LSPCI=`lspci | grep -i qlogic`
        if [ $? = $QL_SUCCESS ]; then
                # check if the installed HBA ISP is one in the
                # input array list
                for LSPCI_LINE in $LSPCI
                do
                        for ISP in "${ISP_LIST[@]}"
                        do
                                echo "$LSPCI_LINE" | grep "$ISP" >& /dev/null
                                if [ $? = $QL_SUCCESS ]; then
                                        # Store it in our array
 					echo "${QL_HBA_LIST[@]}" | grep "$ISP" >& /dev/null
 					if [ $? != 0 ]; then
 						QL_HBA_LIST=( "${QL_HBA_LIST[@]}" "$ISP" );
 					fi
                                        ret_status=$QL_SUCCESS
                                fi
                        done
                done
	else
                # Could not find qlogic device
                # This could be because either `lspci` does
                # not work on this system or QLogic device is
                # not installed on this system.
                # Ask user to force for installation
                ret_status=$QL_FAIL;
        fi
        return $ret_status;

}

# --------------------------------------------------------- #
# detect_all_isp ()	                                    #
# Detects all installed FC HBA                        	    #
# Parameter: None					    # 
# Returns: None						    #
# --------------------------------------------------------- #
function detect_all_isp() {
	# 1 Detect FC
	ISP_LIST=`echo "${QL_FC_ISP_LIST[@]}"`
	detect_isp "$ISP_LIST"
	if [ $? = $QL_SUCCESS ]; then #found FC HBA
		QL_FC_HBA_LIST=( ${QL_HBA_LIST[@]} )
	fi
	return
}

# --------------------------------------------------------- #
# convert_isp_to_modules ()	                            #
# Converts the ISP number to the corresponding Module	    #
# Works on the QL_FC_HBA_LIST and output in  QL_HBA_MODULES #
# Parameter: None	 	                            #
# Returns: None						    # 
# --------------------------------------------------------- #

function convert_isp_to_modules() {
	#clear up the QL_HBA_MODULES array
	QL_HBA_MODULES=( )

	case $KERNEL_MAJ_MIN in
	2.6 )
		for ISP in ${QL_FC_HBA_LIST[@]}
		do
			case $ISP in
			2312 | 6312 )
				ISP=2300
			;;
			6322 )
				ISP=2322
			;;
			2400 | 2432 | 2422 | 5422 | 5432 )
				ISP=2400
			;;
			esac
			
			echo ${QL_HBA_MODULES[@]} | grep "$ISP" >& /dev/null
			if [ $? != 0 ]; then
				QL_HBA_MODULES=( "${QL_HBA_MODULES[@]}" "$ISP" )
			fi
		done
	;;
	2.4 )
		if [ ${#QL_FC_HBA_LIST[@]} -gt 0  ]; then
			QL_HBA_MODULES=( "2300" )
		fi	
	;;
	esac
}

# --------------------------------------------------------- #
# edit_sysconfig ()                                         #
# Function to modify list of modules to be included         #
# in the Ram Disk for SuSE Distribution. Edits the          #
# /etc/sysconfig/kernel                                     #
#                                                           #
# Parameter:                                                #
#       $*    Space separated module list                   #
# Returns: None                                             #
# --------------------------------------------------------- #
function edit_sysconfig() {
        MODLIST="$1"
        # check if Module already in the sysconfig/kernel
        grep -w "$MODLIST" $SLES_CONF >& /dev/null
        if [ $? = 0 ]; then
                # MODLIST already there, quit from here
                return 1
        fi
        MODULES=`cat $SLES_CONF | grep "^INITRD_MODULES" |
                        awk 'BEGIN {FS="="} {print $2}'| sed 's/\"//g'`
        MODULES="$MODULES $MODLIST"
        #echo "New module list: $MODULES"
        sed "s/^INITRD_MODULES.*/INITRD_MODULES=\"$MODULES\"/"  $SLES_CONF>/tmp/kernel
        mv -f /tmp/kernel $SLES_CONF
	echo " $MODLIST" >> $QL_CONF_ENTRIES
	return 0
}

# --------------------------------------------------------- #
# edit_modprobe -- Adds scsi_hostadapter entry in 	    #
# Adds the module in hotplug blacklist to avoid loading     #
# the driver using hotplug				    #
# /etc/modprobe.conf					    #
# Parameter:						    #
# 	$1 Name of the module				    #
# --------------------------------------------------------- #
edit_modprobe () {
	test -z "$1" && return 1

	DONE=0
 	CNT=""
	SCSIHOSTADAPTER="scsi_hostadapter"

   	if test -f "${SLES}" ; then
		MPC=${SLES_MPC}
		return 1
	else
		MPC=${RHEL_MPC}
		# Check if the entry is already in the 
		# /etc/modprobe.conf file
		grep "^[[:space:]]*alias[[:space:]]\+${SCSIHOSTADAPTER}.*${1}$" $MPC >& /dev/null
		if [ $? = 0 ]; then #Entry already in modprobe.conf
			return 1
		fi

		# Get the hostadatper number
		while [ $DONE -eq 0 ]
		do
			TEXT=`cat ${MPC} | grep "^[[:space:]]*alias[[:space:]]\+${SCSIHOSTADAPTER}${CNT}"`
			if [ "$TEXT" = "" ]
				then
					DONE=1
			else
				if [ "$CNT" = "" ]; then
					CNT=0
				fi
				CNT=`expr $CNT + 1`
			fi
		done
		#echo "adding line: alias ${SCSIHOSTADAPTER}${CNT} $1 to $MPC"
		echo "alias ${SCSIHOSTADAPTER}${CNT} $1" >> $MPC
		echo "alias ${SCSIHOSTADAPTER}${CNT} $1" >> $QL_CONF_ENTRIES
	fi
	return 0
}

#-----------------------------------------------------------#
# clear_modprobe											#
# Removes the entries of qla2 from modprobe.conf			#
# and stores the backup in QL_LINE_ENTRIES					#
# Parameters: NONE											#
# Return: 	  NONE											#
#-----------------------------------------------------------#

function clear_modprobe()
{
  MOD_CONF=/etc/modprobe.conf
  COUNT=`grep -c qla2 $MOD_CONF`
  if [ $COUNT -ne 0 ]; then
  	grep "^[[:space:]]*alias[[:space:]]\+scsi_hostadapter.*[[:space:]]\+qla2" $MOD_CONF > $QL_LINE_ENTRIES
	cat $QL_LINE_ENTRIES > /tmp/ql2-entries.txt
	echo "ADDING ENTRIES TO $QL_LINE_ENTRIES" 
  	grep -v "^[[:space:]]*alias[[:space:]]\+scsi_hostadapter.*[[:space:]]\+qla2" $MOD_CONF > temp.txt
     mv -f temp.txt $MOD_CONF
   fi
}


# ---------------------------------------------------------- #
# update_modules_conf()  	                             #
# Updates the moudles configuration file. /etc/modprobe.conf #
# on RHEL and /etc/sysconfig/kernel on SLES                  #
# Parameter: None	 	                             #
#	$1	Module list to be added in modprobe	     #
# Returns: None						     # 
# ---------------------------------------------------------- #
function update_modules_conf() 
{
	MODULE_LIST=(`echo "$1"`)

	if [ "${KERNEL_MAJ_MIN}" = "2.6" ]; then
		if [ ${#MODULE_LIST[@]} -gt 0 ]; then
			if [ -f $SLES ]; then
				#Append $MODULE modules to the list in sysconfig/kernel.
				edit_sysconfig "qla2xxx"
			fi
		fi
	fi

	# Add entry in the /etc/modprobe.conf
	for MODULE in ${MODULE_LIST[@]}
	do

		if [ -f $SLES ]; then
			#Append $MODULE modules to the list in sysconfig/kernel.
			edit_sysconfig "qla$MODULE"
		fi
	done
}

function modify_dkms()
{
	SUPPORTED_MODULES=( 2300 2322 2400 )
	LINE_NUM=""
	DKMS_FILE=/usr/src/${QL_MODULE_VER}/dkms.conf
	TMP_DKMS_FILE=/usr/src/${QL_MODULE_VER}/tmp.dkms.conf
	detect_all_isp
	convert_isp_to_modules

	for MODULE in ${SUPPORTED_MODULES[@]}
	do
		echo "${QL_HBA_MODULES[@]}" | grep "$MODULE" >& /dev/null
		if [ $? -eq 0 ]; then #enable scsi_hostadapter line
			#get the line number
			LINE_NUM=`grep -n "^#qla$MODULE:.*" $DKMS_FILE | sed "s/:.*//"`
			#need to un-comment LINE_NUM
			#check LINE_NUM is valid
			echo "$LINE_NUM" | grep "[[:digit:]]\+" >& /dev/null
			if [ $? -eq 0 ]; then
				cat $DKMS_FILE | sed "${LINE_NUM},${LINE_NUM}s/#qla$MODULE:\(.*\)/\1/" > $TMP_DKMS_FILE
				if [ -f "$TMP_DKMS_FILE" ]; then
					mv -f $TMP_DKMS_FILE $DKMS_FILE
				fi
			fi # else do nothing

		else # not in module list
			if [ -f $SLES ]; then
				#remove from dkms.conf
				#get the line number
				LINE_NUM=`grep -n "^MODULE_NAME.*qla$MODULE.*" $DKMS_FILE | sed "s/:.*//"`
				#need to comment out 5 lines starting from LINE_NUM
				#check LINE_NUM is valid
				echo "$LINE_NUM" | grep "[[:digit:]]\+" >& /dev/null
				if [ $? = 0 ]; then
					NEXT_LINE=`expr $LINE_NUM + 4`
					cat $DKMS_FILE | sed "${LINE_NUM},${NEXT_LINE}s/\(.*\)/#\1/" > $TMP_DKMS_FILE
					if [ -f "$TMP_DKMS_FILE" ]; then
						mv -f $TMP_DKMS_FILE $DKMS_FILE
					fi
				fi # else do nothing
			fi
		fi
	done
}

# --------------------------------------------------------- #
# remove_module_renames()                                   #
# Removes any aliases to the module renames done in the     #
# mdoule-renames files. This is applicable to SLES 10 only  #
# Since on SLES 10,qla2300, qla2400 etc are renamed to 	    #
# qla2xxx						    #
# Parameter: None 		                            #
# Returns: 0.		 				    #
# --------------------------------------------------------- #
function remove_module_renames ()
{
        local BACKUP_DATE=`date +%m%d%y-%H%M%S`
        local TMP_RENAMES=/etc/modprobe.d/renames.ql
        local TMP_RESTORE=${QL_MOD_DIR}/renames.restore.ql
        #if this is SuSe and 2.6
        if [ "${KERNEL_MAJ_MIN}" = "2.6" ] && [ -f $SLES ]; then
                #Take backup of the file once
                if [ -f "${QL_RENAMES_FILE}" ]; then
                        if [ ! -f $QL_RENAMES_FILE_BKUP ]; then
                                cp -f $QL_RENAMES_FILE $QL_RENAMES_FILE_BKUP
                        fi
                        #also take date-wise backup
                        rm -f ${QL_RENAMES_FILE_DATE}-* >& /dev/null
                        cp -f $QL_RENAMES_FILE ${QL_RENAMES_FILE_DATE}-${BACKUP_DATE}
                        #now remove the alias
                        grep "^alias.*qla2xxx$" $QL_RENAMES_FILE | sed "s/^/#/" > $TMP_RESTORE
                        cat $QL_RENAMES_FILE | sed "s/\(^alias.*qla2xxx$\)/#\1/" > $TMP_RENAMES
                        if [ -s "$TMP_RENAMES" ]; then
                                mv -f $TMP_RENAMES $QL_RENAMES_FILE
                        fi

                fi
        fi
}



if [ "$1" = "-modify-dkms-conf" ]; then
	modify_dkms	
	exit 0
fi

if [ "$1" = "-clear-conf" ]; then
		# First of all remove the entries of previous qla2 in modprobe.conf[.local]

		if [ -f $SLES ]; then
			clear_modprobe_local
		else #Assume RHEL
			echo "CLEARING THE MODPROBE.CONF ENTRIES"
			clear_modprobe
		fi
		exit 0
fi

# 1. Detect all the installed HBAs
echo "Detecting installed HBAs..."
detect_all_isp

# 2. Convert the HBAs into module names
convert_isp_to_modules

# 3. Modify modprobe.conf or kernel file
echo "Modifying modules configuration file..."



HBA_LIST="${QL_HBA_MODULES[@]}"
update_modules_conf "$HBA_LIST"

#rename the modules-rename file
remove_module_renames
