#!/bin/sh

# This script removes changes done by post insall script
# Most of the code here is taken from QL installer

QL_SUCCESS=0
QL_FAIL=1

KERNEL_VERSION=`uname -r`
KERNEL_MAJ_MIN=`echo $KERNEL_VERSION | cut -d . -f -2` 

SLES=/etc/SuSE-release
SYS_FILE=kernel
SLES_CONF=/etc/sysconfig/$SYS_FILE
SLES_CONF_KEY=INITRD_MODULES
RHEL=/etc/redhat-release

SLES_MODPROBE="modprobe.conf.local"
SLES_MPC=/etc/$SLES_MODPROBE
RHEL_MODPROBE="modprobe.conf"
RHEL_MPC=/etc/$RHEL_MODPROBE

if [ "${KERNEL_MAJ_MIN}" = "2.4" ]; then
	RHEL_MODPROBE="modules.conf"
	RHEL_MPC=/etc/$RHEL_MODPROBE
fi

QL_MODULE_VER=qla2xxx-8.01.05
QL_CONF_ENTRIES="/usr/src/$QL_MODULE_VER/ql-conf-entries"
QL_OPTIONS_ENTRIES="/usr/src/$QL_MODULE_VER/ql-opt-entries"
QL_LINE_ENTRIES="/usr/src/$QL_MODULE_VER/qla2-entries"

if [ -f $RHEL ]; then

	if [ -f $QL_CONF_ENTRIES ]; then
		grep -v -f $QL_CONF_ENTRIES $RHEL_MPC > /tmp/$RHEL_MODPROBE
		if [ -f /tmp/$RHEL_MODPROBE ]; then
			echo -e "Removing installation changes from $RHEL_MPC...."
			mv -f /tmp/$RHEL_MODPROBE $RHEL_MPC
		else
			echo "Unable to update $RHEL_MPC file"
			return;
		fi
	fi
elif [ -f $SLES ]; then

	#From the "INITRD_MODULES" line in the /etc/sysconfig/kernel 
	# remove entries present in $QL_KERNEL_ENTRIES
	if [ -f "$QL_CONF_ENTRIES" ]; then
		INITRD_LINE=`grep "^INITRD_MODULES" $SLES_CONF`
		
		for MODULES in `cat $QL_CONF_ENTRIES`
		do
			INITRD_LINE=`echo $INITRD_LINE | sed "s/$MODULES//"`
			INITRD_LINE=`echo $INITRD_LINE | sed "s/[[:space:]][[:space:]]\+/ /"`
		done 
		echo -e "Removing installation changes from $SLES_CONF...."
		echo -e ""
		sed "s/^INITRD_MODULES.*/$INITRD_LINE/"  $SLES_CONF > /tmp/kernel
		cp -f /tmp/kernel $SLES_CONF
	fi
fi

function remove_options()
{
# remove the options entries
MOD_CONF=""
if [ -f $SLES ]; then
	MOD_CONF=/etc/modprobe.conf.local
elif [ -f $RHEL ]; then
	MOD_CONF=/etc/modprobe.conf
else
	echo "Distribution not supported..."
	exit 1
fi

FAILOVER="ql2xfailover"
CONFIG_REQ="ConfigRequired"

if [ -f $QL_OPTIONS_ENTRIES ]; then
	#revert the entries
	FAILOVER_VAL=0;
	CONFIG_REQ_VAL=0;
	FAILOVER_VAL=`cat $QL_OPTIONS_ENTRIES | grep $FAILOVER | sed "s/.*${FAILOVER}=\(.\).*/\1/"`
	CONFIG_REQ_VAL=`cat $QL_OPTIONS_ENTRIES | grep $CONFIG_REQ | sed "s/.*${CONFIG_REQ}=\(.\).*/\1/"`
	if [ -f $MOD_CONF ]; then
		OPTION_LINE=`grep "^options[[:space:]]\+qla2xxx.*" $MOD_CONF`

		#revert failover
		if [ "$FAILOVER_VAL" != "" ]; then
			OPTION_LINE=`echo $OPTION_LINE | sed "s/\(.*\)$FAILOVER=.\(.*\)/\1$FAILOVER=$FAILOVER_VAL\2/"`
		else #remove the entry
			OPTION_LINE=`echo $OPTION_LINE | sed "s/\(.*\)$FAILOVER=.\(.*\)/\1\2/"`
			OPTION_LINE=`echo $OPTION_LINE | sed "s/[[:space:]][[:space:]]\+/ /g"`
		fi
		#revert config req
		if [ "$CONFIG_REQ_VAL" != "" ]; then
			OPTION_LINE=`echo $OPTION_LINE | sed "s/\(.*\)$CONFIG_REQ=.\(.*\)/\1$CONFIG_REQ=$CONFIG_REQ_VAL\2/"`
		else #remove the entry
			OPTION_LINE=`echo $OPTION_LINE | sed "s/\(.*\)$CONFIG_REQ=.\(.*\)/\1\2/"`
			OPTION_LINE=`echo $OPTION_LINE | sed "s/[[:space:]][[:space:]]\+/ /g"`
		fi

		cat $MOD_CONF | sed "s/^options.*qla2xxx.*/$OPTION_LINE/" > /tmp/tmp.conf
		if [ -f "/tmp/tmp.conf" ]; then
			mv -f /tmp/tmp.conf $MOD_CONF
		fi
	fi
else
	#remove the failover=0 and configreq=0 entries
	if [ -f $MOD_CONF ]; then
		OPTION_LINE=`grep "^options[[:space:]]\+qla2xxx.*" $MOD_CONF`
		if [ "$OPTION_LINE" = "" ]; then #line not there
			#do nothing
			return
		else
			#check if ql2xfailover already there
			echo $OPTION_LINE | grep "$FAILOVER"  >& /dev/null
			if [ $? = 0 ]; then
			
				echo $OPTION_LINE  | sed "s/.*\($FAILOVER=.\).*/\1/" | grep -i "0" >& /dev/null  
				if [ $? = 0 ]; then
				#remove failover option
					OPTION_LINE=`echo $OPTION_LINE | sed "s/\(.*\)$FAILOVER=.\(.*\)/\1\2/"`
					OPTION_LINE=`echo $OPTION_LINE | sed "s/[[:space:]][[:space:]]\+/ /g"`
				fi
			fi

			#check if ConfigRequired is there or not, then remove it
			echo $OPTION_LINE | grep "$CONFIG_REQ"  >& /dev/null
			if [ $? = 0 ]; then
				
				echo $OPTION_LINE  | sed "s/.*\($CONFIG_REQ=.\).*/\1/" | grep -i "0" >& /dev/null  
				if [ $? = 0 ]; then
				#remove ConfigRequired
				OPTION_LINE=`echo $OPTION_LINE | sed "s/\(.*\)$CONFIG_REQ=.\(.*\)/\1\2/"`
				OPTION_LINE=`echo $OPTION_LINE | sed "s/[[:space:]][[:space:]]\+/ /g"`
				fi
			fi

			# replace the line
			# if option line does not contain anything just remove it
			echo $OPTION_LINE | grep -i "=" >& /dev/null
			if [ $? = 0 ]; then
				
				cat $MOD_CONF | sed "s/^options.*qla2xxx.*/$OPTION_LINE/" > /tmp/tmp.conf
			else
				
				cat $MOD_CONF | grep -v "^options.*qla2xxx.*" > /tmp/tmp.conf
			fi

			if [ -f "/tmp/tmp.conf" ]; then
				mv -f /tmp/tmp.conf $MOD_CONF
			fi
		fi 
	fi
fi
}

if [ "$KERNEL_MAJ_MIN" = "2.6" ]; then
	remove_options
fi

# remove the entries files
if [ -f $QL_CONF_ENTRIES ]; then
	rm -f $QL_CONF_ENTRIES
fi

if [ -f $QL_OPTIONS_ENTRIES ]; then
	rm -f $QL_OPTIONS_ENTRIES
fi

if [ -f $QL_LINE_ENTRIES ]; then
	cat $QL_LINE_ENTRIES >> $MOD_CONF
	rm -f $QL_LINE_ENTRIES
fi

#now do a depmod, on 2.6 RHEL 4 U3 it is not done causing lot of issues
depmod -a >& /dev/null
